/*
 * DinX test program, displays a dragable window
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <dinx/access.h>

#define min(a,b)	((a) < (b) ? (a) : (b))
#define max(a,b)	((a) > (b) ? (a) : (b))

struct test_win {
	int fd;
	unsigned long fgcol;
	unsigned long bgcol;
	dvect pos;
	int buttons;
	drect bounds;
	dvect origin;
	int moving;
};

static void
test_redraw(void *data, const drect *rect, int more)
{
	struct test_win *win = (struct test_win *)data;
	int x, y;

	dinx_fill_rect(win->fd, rect, win->bgcol);
	for (x = win->bounds.x1; x < win->bounds.x2; x += 50) {
		for (y = win->bounds.y1; y < win->bounds.y2; y+= 50) {
			drect r;
			
			r.x1 = max(x, rect->x1);
			r.y1 = max(y, rect->y1);
			r.x2 = min(x + 25, rect->x2);
			r.y2 = min(y + 25, rect->y2);
			dinx_fill_rect(win->fd, &r, win->fgcol);
		}
	}
}

static void
test_mouse(void *data, const dvect *pos, int buttons)
{
	struct test_win *win = (struct test_win *)data;
	int oldbuttons = win->buttons;
	int x, y;
	
	win->buttons = buttons;
	if (!buttons) {
		win->moving = 0;
	}
	/* convert to window coords */
	x = pos->x - win->origin.x;
	y = pos->y - win->origin.y;
	if (!win->moving) {
		if (x < win->bounds.x1 || x >= win->bounds.x2
		 || y < win->bounds.y1 || y >= win->bounds.y2) {
			return;
		}
	}
	if (buttons != oldbuttons) {
		if (buttons & 4) {
			dinx_req_raise(win->fd);
		} else if (buttons & 1) {
			dinx_req_lower(win->fd);
		}
	}
	if (buttons && !oldbuttons) {
		win->moving = 1;
		win->pos.x = x;
		win->pos.y = y;
	}
	if (win->moving) {
		int dx = x - win->pos.x;
		int dy = y - win->pos.y;
		win->origin.x += dx;
		win->origin.y += dy;
		dinx_req_bounds(win->fd, &win->bounds, &win->origin);
	}
}

static void
test_tree_info(void *data, const drect *bounds, int bpp)
{
	struct test_win *win = (struct test_win *)data;

	win->bounds.x1 = 0;
	win->bounds.y1 = 0;
	win->bounds.x2 = rand() % 500;
	win->bounds.y2 = rand() % 500;
	win->origin.x = rand() % bounds->x2;
	win->origin.y = rand() % bounds->y2;
	dinx_req_bounds(win->fd, &win->bounds, &win->origin);
	dinx_req_map(win->fd);
}

struct devent_ops test_ops = {
	test_tree_info,
	NULL,		/* message */
	test_redraw,
	test_mouse,
	NULL		/* key */
};

int main(void)
{
	struct test_win win;
	
	srand(time(NULL));
	win.bgcol = rand();
	win.fgcol = rand();
	win.pos.x = 0;
	win.pos.y = 0;
	win.buttons = 0;
	win.moving = 0;
	win.fd = dinx_open(NULL);
	if (win.fd < 0) {
		perror("dinx_open");
		exit(1);
	}

	while (1) {
		dinx_read_event(win.fd, &test_ops, &win);
	}
	return 0;
}


	
	