/*
 * DinX torture test, opens a window and sends random commands
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is DinX.
 * 
 * The Initial Developer of the Original Code is Ben Williamson.
 * Portions created by Ben Williamson are Copyright (C) 1999
 * Ben Williamson. All Rights Reserved.
 * 
 * Contributor(s):	Ben Williamson <benw@pobox.com>
 * 
 * Alternatively, the contents of this file may be used under the terms
 * of the GNU General Public license (the  "[GPL] License"), in which case the
 * provisions of [GPL] License are applicable instead of those
 * above.  If you wish to allow use of your version of this file only
 * under the terms of the [GPL] License and not to allow others to use
 * your version of this file under the MPL, indicate your decision by
 * deleting  the provisions above and replace  them with the notice and
 * other provisions required by the [GPL] License.  If you do not delete
 * the provisions above, a recipient may use your version of this file
 * under either the MPL or the [GPL] License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <dinx/access.h>

#define min(a,b)	((a) < (b) ? (a) : (b))
#define max(a,b)	((a) > (b) ? (a) : (b))


int myrand(int max)
{
	return (int)((max * 1.0) * rand() / (RAND_MAX + 1.0));
}

#define RAND(x,y) do {x = myrand(1280 + 400) - 200; y = myrand(1024 + 400) - 200; } while (0)


void
req_bounds(int fd)
{
	drect bounds;
	dvect origin;
	int x1, y1, x2, y2;
	
	RAND(x1, y1);
	RAND(x2, y2);
	origin.x = min(x1, x2);
	origin.y = min(y1, y2);
	bounds.x1 = 0;
	bounds.y1 = 0;
	bounds.x2 = max(x1, x2) - origin.x;
	bounds.y2 = max(y1, y2) - origin.y;
	dinx_req_bounds(fd, &bounds, &origin);
}

void
fill_rect(int fd)
{
	int x1, y1, x2, y2;
	drect rect;
	
	RAND(x1, y1);
	RAND(x2, y2);
	rect.x1 = min(x1, x2);
	rect.y1 = min(y1, y2);
	rect.x2 = max(x1, x2);
	rect.y2 = max(y1, y2);
	dinx_fill_rect(fd, &rect, rand());
}
	
void
move_rect(int fd)
{
	int x1, y1, x2, y2;
	drect rect;
	dvect moveby;
	
	RAND(x1, y1);
	RAND(x2, y2);
	rect.x1 = min(x1, x2);
	rect.y1 = min(y1, y2);
	rect.x2 = max(x1, x2);
	rect.y2 = max(y1, y2);
	RAND(moveby.x, moveby.y);
	dinx_move_rect(fd, &rect, &moveby);
}


int main(void)
{
	int fd = -1;
	
	srand(time(NULL));
	while (1) {
		int i = myrand(3000);
		
		if (fd < 0) {
			fd = dinx_open(NULL);
		}
		fill_rect(fd);
		if (i < 3) {
			dinx_close(fd);
			fd = -1;
		} else if (i < 10) {
			dinx_req_unmap(fd);
		} else if (i < 500) {
			dinx_req_map(fd);
		} else if (i < 1000) {
			move_rect(fd);
		} else if (i < 2000) {
			req_bounds(fd);
		}
	}
}

